@@ -189,14 +189,13 @@ from abc import abstractmethod, ABC
 from copy import deepcopy
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
-from ae.inspector import stack_vars                                                                       # type: ignore
+from ae.inspector import stack_vars, try_eval                                                             # type: ignore
 from ae.i18n import default_language, get_f_string, get_text, register_package_translations, translation  # type: ignore
 from ae.gui_app import (                                                                                  # type: ignore
-    FLOW_KEY_SEP, flow_action, flow_class_name, flow_path_id, id_of_flow, module_globals, update_tap_kwargs,
-    MainAppBase)
+    FLOW_KEY_SEP, flow_action, flow_class_name, flow_path_id, id_of_flow, update_tap_kwargs, MainAppBase)
 
 
-__version__ = '0.1.25'
+__version__ = '0.1.27'
 
 
 register_package_translations()
@@ -213,8 +212,6 @@ IGNORED_HELP_STATES = ('flow_id', 'flow_path', 'win_rectangle')     #: tuple of
 
 REGISTERED_TOURS = dict()                                           #: map(name: class) of all registered tour classes
 
-HIDDEN_GLOBALS = ('__builtins__', '__doc__', 'module_globals')      #: globals that get removed from help_variables
-
 
 PsHintsType = Dict[str, Any]                                        #: actually Dict[str, Union[float, str]] but mypy...
 
@@ -291,13 +288,13 @@ def id_of_flow_help(flow_id: str) -> str:
     return f'{FLOW_HELP_ID_PREFIX}{flow_id}'
 
 
-def id_of_state_help(state_name: str) -> str:
+def id_of_state_help(app_state_name: str) -> str:
     """ compose help id for app state name/key.
 
-    :param state_name:      name of the app state variable.
+    :param app_state_name:  name of the app state variable.
     :return:                help id for the specified app state.
     """
-    return f'{APP_STATE_HELP_ID_PREFIX}{state_name}'
+    return f'{APP_STATE_HELP_ID_PREFIX}{app_state_name}'
 
 
 def id_of_tour_help(page_id: str) -> str:
@@ -342,9 +339,9 @@ def layout_ps_hints(wid_x: float, wid_y, wid_width, wid_height, win_width, win_h
     else:
         anchor_dir_y = 'i'
     if max_width > max_height:
-        anchor_dir = anchor_dir_x
-        anchor_x = wid_x + (0 if anchor_dir_x == 'l' else wid_width)
-        anchor_y = wid_y + wid_height / 2
+        anchor_dir = anchor_dir_x                                       # pylint: disable=possibly-unused-variable
+        anchor_x = wid_x + (0 if anchor_dir_x == 'l' else wid_width)    # pylint: disable=possibly-unused-variable
+        anchor_y = wid_y + wid_height / 2                               # pylint: disable=possibly-unused-variable
         max_height = win_height
     else:
         anchor_dir = anchor_dir_y
@@ -395,6 +392,16 @@ def register_tour_class(tour_class: Type['TourBase']):
     REGISTERED_TOURS[tour_class.__name__] = tour_class
 
 
+def tour_help_translation(page_id: str) -> Optional[Union[str, Dict[str, str]]]:
+    """ determine help translation for the passed page id (flow id or app state name).
+
+    :param page_id:         tour page id.
+    :return:                help translation text/dict (if exists) or None if translation not found.
+    """
+    return (translation_short_help_id(id_of_flow_help(page_id))[0] or
+            translation_short_help_id(id_of_state_help(page_id))[0])
+
+
 def tour_id_class(tour_id: str) -> Optional[Any]:
     """ determine the tour class of the passed tour id.
 
@@ -463,16 +470,20 @@ class HelpAppBase(MainAppBase, ABC):
         :param widget:          the popup/dropdown/container widget to be moved to the top.
         """
 
+    @abstractmethod
+    def help_activation_toggle(self):
+        """ button tapped event handler to switch help mode between active and inactive (also inactivating tour). """
+
     # overwritten methods
 
-    def change_app_state(self, state_name: str, state_value: Any, send_event: bool = True, old_name: str = ''):
+    def change_app_state(self, app_state_name: str, state_value: Any, send_event: bool = True, old_name: str = ''):
         """ change app state via :meth:`~ae.gui_app.MainAppBase.change_app_state`, show help text in active help mode.
 
         All parameters are documented in the overwritten method :meth:`~ae.gui_app.MainAppBase.change_app_state`.
         """
-        help_vars = dict(state_name=state_name, state_value=state_value, send_event=send_event, old_name=old_name)
+        help_vars = dict(app_state_name=app_state_name, state_value=state_value, old_name=old_name)
         if not self.help_app_state_display(help_vars):
-            super().change_app_state(state_name, state_value, send_event=send_event, old_name=old_name)
+            super().change_app_state(app_state_name, state_value, send_event=send_event, old_name=old_name)
             self.help_app_state_display(help_vars, changed=True)
 
     def change_flow(self, new_flow_id: str, **event_kwargs) -> bool:
@@ -511,11 +522,11 @@ class HelpAppBase(MainAppBase, ABC):
         :param changed:         False before change of the app state, pass True if app state got just/already changed.
         :return:                True if help mode and layout is active and found target widget is locked, else False.
         """
-        state_name = help_vars.get('state_name')
-        if not state_name or state_name in IGNORED_HELP_STATES:
+        app_state_name = help_vars.get('app_state_name')
+        if not app_state_name or app_state_name in IGNORED_HELP_STATES:
             return False
 
-        help_id = id_of_state_help(state_name)
+        help_id = id_of_state_help(app_state_name)
 
         hlw = self.help_layout
         if hlw is None:
@@ -556,7 +567,7 @@ class HelpAppBase(MainAppBase, ABC):
             help_id = short_help_id = has_trans['next_help_id']     # type: ignore # silly mypy, Pycharm is more clever
             key_suffix = ''
 
-        glo_vars, help_vars = self.help_variables(help_vars)
+        glo_vars = self.global_variables()
         hlw: Any = self.help_layout
         hlw.tip_text = get_f_string(short_help_id, key_suffix=key_suffix, glo_vars=glo_vars, loc_vars=help_vars)
         hlw.explained_widget = self.help_widget(help_id, help_vars)     # set help target widget
@@ -620,21 +631,6 @@ class HelpAppBase(MainAppBase, ABC):
             help_id = ''
         return target, help_id
 
-    def help_variables(self, help_vars: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:
-        """ determine globals and extend help vars for help text evaluation/translation
-
-        :param help_vars:       initial help variable locals that will be extended and returned.
-        :return:                global and local variable dicts.
-        """
-        glo_vars = {k: v for k, v in module_globals.items() if k not in HIDDEN_GLOBALS}
-        glo_vars.update((k, v) for k, v in globals().items() if k not in HIDDEN_GLOBALS)
-        glo_vars['_add_base_globals'] = True        # instruct ae.inspector.try_eval to add ae.core globals
-
-        help_vars['app'] = self.framework_app
-        help_vars['main_app'] = self
-
-        return glo_vars, help_vars
-
     def help_widget(self, help_id: str, help_vars: Dict[str, Any]) -> Any:
         """ ensure/find help target widget via attribute name/value and extend :paramref:`~help_widget.help_vars`.
 
@@ -681,7 +677,7 @@ class HelpAppBase(MainAppBase, ABC):
 
     def on_app_started(self):
         """ last app init event on app startup. """
-        self.starting_tour()
+        self.start_app_tour()
 
     def on_flow_popup_close(self, _flow_key: str, _popup_args: Dict[str, Any]) -> bool:
         """ overwritten popup close handler of FlowPopup widget to reset help widget/text. """
@@ -695,7 +691,7 @@ class HelpAppBase(MainAppBase, ABC):
             self.tour_layout.stop_tour()
         return super().save_app_states()
 
-    def starting_tour(self, tour_class: Optional[Type['TourBase']] = None) -> bool:
+    def start_app_tour(self, tour_class: Optional[Type['TourBase']] = None) -> bool:
         """ start new app tour, automatically cancelling a currently running app tour.
 
         :param tour_class:          optional tour (pages) class, default: tour of current help id or `OnboardingTour`.
@@ -707,7 +703,7 @@ class HelpAppBase(MainAppBase, ABC):
                 self.tour_layout.stop_tour()
             tour_layout_class(self, tour_class=tour_class)  # pylint: disable=not-callable # false positive
         else:
-            self.vpo("HelpAppBase.starting_tour called but tour overlay/layout class got not set")
+            self.vpo("HelpAppBase.start_app_tour called but tour overlay/layout class got not set")
 
         return bool(self.tour_layout)   # overlay instance sets main_app./framework_app.tour_layout on tour start
 
@@ -739,7 +735,7 @@ class TourBase:
     of your tour-specific sub-class on tour start.
     """
     auto_switch_pages: bool = False
-    """ set to True for to automatically switch tour pages within the seconds specified by page_data['next_page_delay'].
+    """ if True switch automatically tour pages within the seconds specified by :attr:`.page_data`['next_page_delay'].
     """
 
     page_data: Dict[str, Any] = dict(help_vars=dict())
@@ -750,18 +746,21 @@ class TourBase:
 
     supported/implemented dict keys:
 
-    (empty string)      tour page fstring message text template.
-    app_flow_delay      time in seconds to wait until app flow change is fully completed (def=1.2, >0.9 for auto-width).
-    app_reset_delay     time in seconds between app reset/close-popups and flow reset/prepare-first-tour-page (def=0.3).
-    back                caption of tour previous page button (def='back').
-    help_vars           additional help variables, e.g. `help_translation` providing context help text.
-    next                caption of tour next page button (def='next').
-    next_page_delay     time in seconds to read the current tour page before next auto_switch_next_page() (def=9.6).
+    * `app_flow_delay`: time in seconds to wait until app flow change is fully completed (def=1.2, >0.9 for auto-width).
+    * `app_reset_delay`: time in seconds between app reset/close-popups and flow reset/prepare-tour-page (def=0.3).
+    * `back_text`: caption of tour previous page button (def=get_text('back')).
+    * `help_vars`: additional help variables, e.g. `help_translation` providing context help translation dict/text.
+    * `next_text`: caption of tour next page button (def=get_text('next')).
+    * `next_page_delay`: time in seconds to read the current tour page before next auto_switch_next_page() (def=9.6).
+    * `tip_text` or (empty string): tour page tooltip text fstring message text template.
     """
 
     page_ids: List[str] = list()
     """ tour page ids, either initialized via this class attribute or dynamically. """
 
+    pages_explained_matcher: Dict[str, Union[Callable[[Any], bool], str]] = dict()
+    """ :meth:`MainAppBase.find_widget` match callable to determine explained widget of each tour page. """
+
     def __init_subclass__(cls, **kwargs):
         """ register tour class; called on declaration of tour subclass. """
         super().__init_subclass__(**kwargs)
@@ -776,9 +775,9 @@ class TourBase:
         assert main_app.tour_layout
         self.layout = main_app.tour_layout
 
-        self.page_data = deepcopy(self.page_data)   # copy defaults to instance attr, overwritten via load_page_data()
         self.page_ids = self.page_ids.copy()        # copy page ids from class to instance attribute
         self.page_idx = 0
+        self.came_back = False
 
         self.backup_app_states()
 
@@ -809,11 +808,9 @@ class TourBase:
         page_cnt = len(self.page_ids)
         assert 0 <= page_idx < page_cnt, f"page_idx ({page_idx}) has to be greater zero and less than {page_cnt}"
         page_id = self.page_ids[page_idx]
-        page_data = dict(help_vars=dict())
+        page_data = deepcopy(self.__class__.page_data)        # reset to tour class default: dict(help_vars=dict(), ...)
 
-        help_translation = translation_short_help_id(id_of_flow_help(page_id))[0]
-        if not help_translation:
-            help_translation = translation_short_help_id(id_of_state_help(page_id))[0]
+        help_translation = tour_help_translation(page_id)
         tour_translation = translation_short_help_id(id_of_tour_help(page_id))[0]
         if help_translation:
             if tour_translation:
@@ -821,13 +818,15 @@ class TourBase:
             else:
                 tour_translation = help_translation
 
-        page_data.update(tour_translation if isinstance(tour_translation, dict) else {'': tour_translation})
+        page_data.update(tour_translation if isinstance(tour_translation, dict) else {'tip_text': tour_translation})
 
         self.main_app.vpo(f"TourBase.load_page_data(): tour page{page_idx}/{page_cnt} id={page_id} data={page_data}")
         self.page_data = page_data
 
     def next_page(self):
         """ switch to next tour page. """
+        self.came_back = False
+        self.teardown_page()
         self.setup_next_page()
 
         ids = self.page_ids
@@ -841,7 +840,10 @@ class TourBase:
 
     def prev_page(self):
         """ switch to previous tour page. """
+        self.came_back = True
         self.auto_switch_cancel()
+        self.teardown_page()
+
         self.setup_prev_page()
 
         assert self.page_idx > 0, f"TourBase.prev_page wrong/missing page {self.__class__.__name__}:{self.page_idx - 1}"
@@ -852,6 +854,16 @@ class TourBase:
 
         self.setup_page()
 
+    def remove_page(self, page_id: str):
+        """ remove tour page in teardown page process.
+
+        :param page_id:         id of the tour page to remove from page ids.
+        """
+        if page_id in self.page_ids:
+            self.page_ids.remove(page_id)
+            if not self.came_back:              # if user is leaving the removed page - moving to the next one
+                self.page_idx -= 1              # .. then correct page index
+
     def reset_app_flow(self):
         """ delayed reset of flow path/id for to be processed after dispatch of popups dismiss. """
         self.main_app.vpo("TourBase.reset_app_flow")
@@ -878,40 +890,33 @@ class TourBase:
 
     def setup_explained_widget(self):
         """ determine and set the explained widget for the actual tour page. """
+        main_app = self.main_app
         page_id = self.page_ids[self.page_idx]
-        wid = self.main_app.widget_by_flow_id(page_id)
-        if not wid:
-            wid = self.main_app.widget_by_state_name(page_id)
+        if page_id in self.pages_explained_matcher:
+            matcher = self.pages_explained_matcher[page_id]
+            if isinstance(matcher, str):
+                matcher = try_eval(matcher, glo_vars=main_app.global_variables(layout=self.layout, tour=self))
+            wid = main_app.find_widget(matcher)
+            self.page_data['help_vars']['help_translation'] = tour_help_translation(widget_page_id(wid))
+        else:
+            wid = main_app.widget_by_flow_id(page_id)
+            if not wid:
+                wid = main_app.widget_by_app_state_name(page_id)
         if wid:
             self.layout.explained_widget = wid
 
     def setup_layout(self):
         """ setup/prepare tour overlay/layout after switch of tour page. """
-        self.setup_page_ids()
-
-        page_idx = self.page_idx
-        page_cnt = len(self.page_ids)
-        assert 0 <= page_idx < page_cnt, f"page_idx ({page_idx}) has to be positive and less than {page_cnt}"
-        page_data = self.page_data
-        layout = self.layout
         main_app = self.main_app
-
-        main_app.vpo(f"TourBase.setup_layout(): page{page_idx}/{page_cnt} id={self.page_ids[page_idx]} layout={layout}")
+        layout = self.layout
 
         main_app.ensure_top_most_z_index(layout)
 
+        self.setup_page_ids()
+        main_app.vpo(f"TourBase.setup_layout(): page id={self.page_ids[self.page_idx]}")
+
         self.setup_explained_widget()
-
-        help_vars = page_data['help_vars']
-        help_vars['self'] = layout.explained_widget
-        glo_vars, help_vars = main_app.help_variables(page_data['help_vars'])
-
-        layout.title_text = self.tour_text(page_data.get('title_text'), glo_vars, help_vars)
-        layout.page_text = self.tour_text(page_data.get('page_text'), glo_vars, help_vars)
-        layout.tip_text = self.tour_text(page_data.get(''), glo_vars, help_vars)
-        layout.next_text = page_data.get('next', get_text('next')) if page_idx + 1 < page_cnt else ""
-        layout.prev_text = page_data.get('back', get_text('back')) if page_idx > 0 else ""
-
+        self.setup_texts()
         layout.page_updated()
 
         if self.auto_switch_pages:
@@ -924,7 +929,9 @@ class TourBase:
     def setup_page(self):
         """ setup tour overlay to load data for to display new tour page. """
         self.main_app.vpo(f"TourBase.setup_page page_data={self.page_data}")
+
         self.load_page_data()
+
         app_flow_delay = self.page_data.get('app_flow_delay', 1.2)  # > 0.9 to complete auto width animation
         self.main_app.call_method_delayed(app_flow_delay, self.setup_layout)
 
@@ -936,31 +943,51 @@ class TourBase:
         """ setup app flow to display the previous tour page. """
         self.main_app.vpo("TourBase.setup_prev_page")
 
+    def setup_texts(self):
+        """ setup texts in tour layout from page_data. """
+        main_app = self.main_app
+        layout = self.layout
+        page_data = self.page_data
+        page_idx = self.page_idx
+
+        glo_vars = main_app.global_variables(layout=layout, tour=self)
+        help_vars = page_data['help_vars']
+        help_vars['self'] = layout.explained_widget
+
+        main_app.vpo(f"TourBase.setup_texts page_data={page_data}")
+
+        _txt = lambda _t: _t is not None and get_f_string(_t, glo_vars=glo_vars, loc_vars=help_vars) or ""  # noqa: E731
+
+        layout.title_text = _txt(page_data.get('title_text'))
+        layout.page_text = _txt(page_data.get('page_text'))
+
+        tip_text = page_data.get('tip_text', page_data.get(''))
+        if tip_text is None:
+            help_tra = help_vars.get('help_translation')
+            tip_text = help_tra.get('', "") if isinstance(help_tra, dict) else help_tra
+        layout.tip_text = _txt(tip_text)
+
+        layout.next_text = page_data.get('next_text', get_text('next')) if page_idx + 1 < len(self.page_ids) else ""
+        layout.prev_text = page_data.get('back_text', get_text('back')) if page_idx > 0 else ""
+
     def start(self):
         """ prepare app for tour start. """
         self.main_app.vpo("TourBase.start")
         self.reset_app_states()
         self.page_idx = 0
+        self.came_back = False
         self.setup_page()
 
     def stop(self):
         """ stop/cancel tour. """
         self.main_app.vpo("TourBase.stop")
         self.auto_switch_cancel()
+        self.teardown_page()
         self.restore_app_states()
 
-    @staticmethod
-    def tour_text(text: str, glo_vars: Dict[str, Any], loc_vars: Dict[str, Any]) -> str:
-        """ load, translate and f-string-evaluate text of tour layout/overlay child widget.
-
-        :param text:            text to load, translate and f-string-evaluate.
-        :param glo_vars:        global variables.
-        :param loc_vars:        local variables.
-        :return:                text of translation/help/tour id.
-        """
-        if text is None:
-            return ""
-        return get_f_string(text, glo_vars=glo_vars, loc_vars=loc_vars) or ""
+    def teardown_page(self):
+        """ restore tour page state before switch to prev/next page or tour finishing. """
+        self.main_app.vpo("TourBase.teardown_page")
 
 
 class TourDropdownFromButton(TourBase):
@@ -985,32 +1012,65 @@ class TourDropdownFromButton(TourBase):
 
     def setup_page_ids(self):
         """ prepare all tour pages - first page is explaining opening dropdown button. """
+        super().setup_page_ids()
         page_ids = self.page_ids
         if self.page_idx == 1 and len(page_ids) == 2 and page_ids[1] == TourDropdownFromButton.determine_page_ids:
+            popups = self.main_app.popups_opened()
+            if not popups:
+                self.main_app.po("TourDropDownFromButton.setup_page_ids: dropdown not opened")
+                return
+
+            children = self.main_app.widget_tourable_children_page_ids(popups[-1])
+            if not children:
+                self.main_app.po(f"TourDropDownFromButton.setup_page_ids missing tour-able child in {popups[-1]}")
+                return
+
             page_ids.remove(TourDropdownFromButton.determine_page_ids)
-
-            top_most_popup = self.main_app.popups_opened()[-1]
-            children = self.main_app.widget_tourable_children_page_ids(top_most_popup)
-            if children:
-                page_ids.extend(children)
-                self.load_page_data()   # reload page data of 2nd tour page (explaining the first dropdown child)
-            else:
-                self.main_app.po(f"TourDropDownFromButton.setup_page_ids missing tour-able child in {top_most_popup}")
-
-        super().setup_page_ids()
+            page_ids.extend(children)
+            self.load_page_data()   # reload page data of this 2nd tour page (explaining the first dropdown child)
 
 
 # ====== app tours =============================================================
 
+_OPEN_USER_PREFERENCES_FLOW_ID = id_of_flow('open', 'user_preferences')
+
 
 class OnboardingTour(TourBase):
     """ onboarding tour for first app start. """
-    page_ids = ['', 'help_activate', 'tooltip', 'start_tour', 'explained_widget', 'switch_page', 'stop_tour']
+    page_ids = ['', 'switch_page', 'help_activate', 'tooltip', 'responsible_layout', 'layout_font_size', 'stop_tour']
+
+    pages_explained_matcher = dict(
+        help_activate=lambda widget: widget.__class__.__name__ == 'HelpToggler',
+        tooltip=lambda widget: getattr(widget, 'tap_flow_id', None) == _OPEN_USER_PREFERENCES_FLOW_ID,
+        layout_font_size=lambda widget: getattr(widget, 'app_state_name', None) == 'font_size',
+    )
+
+    def setup_page(self):
+        """ overridden to open user preferences dropdown in responsible_layout tour page. """
+        super().setup_page()
+        page_id = self.page_ids[self.page_idx]
+
+        if page_id == 'layout_font_size':
+            main_app = self.main_app
+            flow_id = _OPEN_USER_PREFERENCES_FLOW_ID
+            wid = main_app.widget_by_flow_id(flow_id)
+            main_app.change_flow(flow_id, **update_tap_kwargs(wid))
+
+    def teardown_page(self):
+        """ overridden to close the opened user preferences dropdown on leaving responsible_layout tour page. """
+        super().teardown_page()
+        page_id = self.page_ids[self.page_idx]
+
+        if page_id == 'switch_page':
+            self.remove_page('switch_page')
+        elif page_id == 'layout_font_size':
+            popups = self.main_app.popups_opened()
+            if popups:
+                popups[-1].close()
 
 
 class UserPreferencesTour(TourDropdownFromButton):
     """ user preferences menu tour. """
     auto_switch_pages = True
-    # resulting in page_data float val: page_data = deepcopy(TourDropdownFromButton.page_data)['next_page_delay'] = 3.6
     page_data = dict(help_vars=dict(), next_page_delay=3.6)
-    page_ids = [id_of_flow('open', 'user_preferences'), TourDropdownFromButton.determine_page_ids]
+    page_ids = [_OPEN_USER_PREFERENCES_FLOW_ID, TourDropdownFromButton.determine_page_ids]
